

import Foundation
import SwiftUI
import DouShouQiModel

class VMGame : ObservableObject {

    @Published var msg: String = ""
    
    var gameScene: GameScene = GameScene(size: CGSize(width: 940, height: 740),player1Name:"TOTO",player1Image: Image("profil"),player2Name:"toto2",player2Image: Image("profil"))

    func start() async {
        defineListener()
        try! await gameScene.game.start()
    }

    func defineListener(){
        gameScene.game.addGameStartedListener { board in self.startGame() }
        gameScene.game.addGameOverListener { board, result, player in self.gameOver(result : result) }
        gameScene.game.addGameChangedListener { game in self.gameChange() }
        gameScene.game.addBoardChangedListener { board in self.boardChange() }

        gameScene.game.addMoveChosenCallbacksListener { board, move, player in self.moveChose(board: board, move: move, player: player) }
        gameScene.game.addInvalidMoveCallbacksListener { board, move, player, bool in self.invalidMove(board: board, move: move, player: player, bool: bool) }

        gameScene.game.addPieceRemovedListener { _, _, piece in self.removePiece(piece: piece) }

        gameScene.game.addPlayerNotifiedListener { board, player in
            print("Player notif : \(player.id) à toi de jouer ")
            self.msg = "Player notif : \(player.id) à toi de jouer !"

            if player is IAPlayer {
                try! await player.chooseMove(in: board, with: self.gameScene.game.rules)
            }
        }
    }

    func startGame() {
        print("Start game !")
        msg = "Start !!"
    }

    func gameOver(result : Result) { print("Game over !")
        msg = "Game over ! "
        switch result
        {
        case .winner(let winner, _):
            switch winner {
            case .player1:
                msg = msg + "player 1 wins !"
            case .player2:
                msg = msg + "player 2 wins !"
            default:
                msg = msg + "equality !"
            }
        default:
            msg="Game over !"
        }
        
        gameScene.isGameOver = true
        actionEndGame(scene: gameScene,message: msg)
    }

    func gameChange() {
        print("Game change !")
    }

    func boardChange() {
        print("Board change !")
        gameScene.displayBoard(board: gameScene.game.board)
    }

    func moveChose(board: Board, move: Move, player: Player) {
    }

    func invalidMove(board: Board, move: Move, player: Player, bool: Bool) {
        if bool {
            print("Move valid de \(player.id)")
            print("Move : \(move.description)")
        } else {
            print("Move invalid de \(player.id)")
            print("Move : \(move.description)")
            gameScene.displayBoard(board: gameScene.game.board)
        }
        print("------------")
    }

    func removePiece(piece:Piece){
        print("Remove piece")
        if let node = gameScene.pieces[piece.owner]![piece.animal] {
            actionRemove(size: gameScene.size, node: node, scene: gameScene)
            print("Remove piece from parent")
        }
    }
}
